/* ----------------------------------------------------------------------
   The SINUS Firewall -- a TCP/IP packet filter for Linux

   Written within the SINUS project at the University of Zurich,
   SWITCH, Telekurs Payserv AG, ETH Zurich.
   originally based on the sf Firewall Software (C) 1996 by Robert
   Muchsel and Roland Schmid.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   SINUS Firewall resources:
   SINUS Homepage: http://www.ifi.unizh.ch/ikm/SINUS/
   Firewall Homepage: http://www.ifi.unizh.ch/ikm/SINUS/firewall.html
   Frequently asked questions: http://www.ifi.unizh.ch/ikm/SINUS/sf_faq.html
   Mailing list for comments, questions, bug reports: firewall@ifi.unizh.ch
   -------------------------------------------------------------------------



	 MD5 Algorithm Copyright Notice:

	 Copyright (C) 1990-2, RSA Data Security, Inc. Created 1990. All
	 rights reserved.

	 RSA Data Security, Inc. makes no representations concerning either
	 the merchantability of this software or the suitability of this
	 software for any particular purpose. It is provided "as is"
	 without express or implied warranty of any kind.

	 These notices must be retained in any copies of any part of this
	 documentation and/or software.

   ----------------------------------------------------------------------  */


/*
 * sf Account - User accounting admninistration tool
 */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <unistd.h>
#include <pwd.h>
#include "sf_custom.h"
#include "sf_relay.h"
#include "sf_md5.h"

static void AddUser PROTO_LIST ((char *));
static void DelUser PROTO_LIST ((char *));
static void QueryPasswd PROTO_LIST ((void));
static void Help PROTO_LIST ((void));

struct 		auth_data passwdbuf;    	// passwd entry
MD5_CTX 	context;									// md5 algorithm context
unsigned 	char digest[16];					// md5 digest
static char login[LOGIN_MAXLEN+1];	// loginbuffer
static char password[20+1];					// new password to set


static char *help =
"Arguments (only one argument at a time):
  -a          - add a new user 
  -d          - delete existing user
  -q          - querie firewall.passwd 
  -h --help   - display help message
  (none)      - display help message
";																	// the help message


int main (int argc, char *argv[]) {

  if (argc > 1) {
   if (strcmp(argv[1], "-a") == 0)
     AddUser();
   else if (strcmp(argv[1], "-d") == 0)
     DelUser();
   else if (strcmp(argv[1], "-q") == 0)
     QueryPasswd();
	 else Help();
	}
	else
	  Help();

  return (0);
}


int seekLogin(struct auth_data *passwdbuf, FILE *passwd, char *login)
{
int i;

	i=0;
  // check for login, key in firewall.passwd
  while (fread(passwdbuf, sizeof(struct auth_data), 1, passwd) == 1) {
		i++;
    if (passwdbuf->login[strlen(login)] == ' ') {
      passwdbuf->login[strlen(login)] = 0;      // terminate loginstr
      if (strcmp(passwdbuf->login, login) == 0)
        return i;
    }
  }
  return 0;
} // seekLogin



int scanLogin(char *prompt)
{
FILE 	*pwd;			// passwd filehandle
int 	result;

	SCANLOGIN:
	printf(prompt);
	printf("No whitespaces allowed. Type ?<CR> for a list of existing logins.\n");
	printf("Login: ");
  scanf("%s20", login);

	// we do not accept trailing '?' in logins
	if (login[0] == '?') {
		QueryPasswd();
		goto SCANLOGIN;
	}
	
	pwd = fopen(PASSWD, "r");
  if (pwd == 0) {
    printf("Error opening passwd file: %s\n", PASSWD);
		return -1;
	}
  else {

    // position stream at the beginning of passwd
    rewind(pwd);

    // check if login already exists
    printf("\nChecking passwd for login \"%s\"... ", login);
    fflush(stdout);       // flush stdout to get message before seeking
    result = seekLogin(&passwdbuf, pwd, login);

		if (pwd != 0)
			fclose(pwd);

		return result;
	}
} // getLogin
 

int CheckPosition(FILE *passwd) {

int i;

	// check if position is valid
	i = ftell(passwd);
	if ((i % sizeof(struct auth_data)) != 0) {
		printf("Error: passwd possibly corrupted!\n Exiting.\n");
		return -1;
	}
	return i;

} // CheckPosition

 
static void AddUser(void) {

FILE   		*passwd;									// firewall passwd
char *check;													// temporary password buffer
unsigned int keynum;									// number of key to generate
unsigned int i, j;										// loop variables
int result;
unsigned char key[MDKEY_SIZE+1];			// final key to set


	// scan for login to add
	printf("\nAdd a new login:\n");
	printf("----------------\n\n");

	result = scanLogin("Enter login to add.\n");
	if (result < 0)
		return;
	if (result == 1) {
		printf("Login already exists.\nExiting.\n\n");
		return;
	}
	else {
		printf("OK\n\n");

		// OK, now scan for keynumber to generate
		printf("Now enter keynumber to generate for login \"%s\".\n", login);
		keynum = 10000;
			
		// loop until we got a valid keynumber < "9999"
		while (keynum > 9999) {
			printf("(Maximum of 9999): ");
			scanf("%u4", &keynum);
		}

		// get the new password
		check = getpass("\nPlease enter new password to be set: ");
		strncpy(password, check, 20);
		password[20] = 0;		// asure NULL-termination
		check = getpass("Please retype new password: ");

		if (strcmp(password, check) != 0) {
			printf("\nPasswords do not match!\nExiting.\n\n");
			return;
		}
		printf("\nCalculating key number %d... ", keynum);
		fflush(stdout); 			// flush stdout to get message before calculating

		strcpy(key, password);

		// now calculate the key
		for (i=keynum + 1; i>0; i--) {	
			MD5Init(&context);
  		MD5Update(&context, key, strlen(key));
  		MD5Final(digest, &context);

			// convert into 32 character hex string
  		for (j=0; j<16; j++)
 				sprintf(&key[j<<1], "%02x", digest[j]); 
		}

		printf("Done.\n");
		printf("Adding login %s... ", login);
		fflush(stdout);

		// Insert login into passwd
		passwd = fopen(PASSWD, "a+");
		if (passwd == 0) {
			printf("Error opening passwd file: %s\n", PASSWD);
			return;
		}
		if (fseek(passwd, 0, SEEK_END) != 0) {
			printf("Error writing to passwd file.\nExiting.\n");
			goto CLEANUP;
		}

		// check if position is valid
		if (CheckPosition(passwd) < 0)
			goto CLEANUP;

		// prepare passwdbuf
		sprintf(passwdbuf.keynum, "%4d", keynum);
		passwdbuf.kspace = ' ';
		strncpy(passwdbuf.key, key, MDKEY_SIZE);
		passwdbuf.lspace = ' ';
		sprintf(passwdbuf.login, "%-20s", login);
		passwdbuf.cr = 10;
	
		// finally write data
		if (fwrite(&passwdbuf, sizeof(struct auth_data), 1, passwd) != 1) {
			printf("Error writing data to passwd!\nExiting.\n");
			goto CLEANUP;
		}
		printf("Done.\n\n");
		printf("***\n");
		printf("*** Next key for login \"%s\" will be %d\n", login, keynum);
		printf("***\n");
	}
	
	CLEANUP:
	if (passwd)
		fclose(passwd);
	return;

} // AddUser



static void DelUser(void) {

int loginPosition;
int passwdSize;
void *passwdMem;
FILE	*passwd;
char	c[2];

	printf("\nDelete a login:\n");
	printf("--------------\n\n");

	login[0] = 0;
	loginPosition = 0;
	while(strlen(login) == 0) {
		loginPosition = scanLogin("Enter login to be deleted.\n");

		if (loginPosition < 0)
			// Error opening passwd file
			return;
		if (loginPosition == 0) {
			// no match
			printf("No such login \"%s\".\nExiting.\n\n", login);
			return;
		}
	}

	printf("\nOK to delete login \"%s\" (y/n)?", login);
	scanf("%1s", c);
	if (c[0] != 'y') {
		printf("Abort by user.\nExiting.\n");
		return;
	}

	passwd = fopen(PASSWD, "r+");
	if (passwd == 0) {
		printf("Error opening %s.\nExiting.\n", PASSWD);
		return;
	}

	// ftell will give us sizeof(file)
	fseek(passwd, 0, SEEK_END);
	passwdSize = (int)ftell(passwd);
	if (passwdSize == -1) {
		printf("Error writing data to passwd file.\nExiting.\n");
		goto CLEANUP;
	}

	passwdMem = malloc(passwdSize);
	if (passwdMem == 0) {
		printf("Can't allocate memory.\nExiting.\n");
		free(passwdMem);
		goto CLEANUP;
	}

	rewind(passwd);
	if (fread(passwdMem, passwdSize, 1, passwd) != 1) {
		printf("Error reading passwd file.\nExiting.\n");
		free(passwdMem);
		goto CLEANUP;
	}

	memmove(passwdMem+((loginPosition-1)*(sizeof(struct auth_data))),
			passwdMem+((loginPosition)*(sizeof(struct auth_data))),
			passwdSize-((loginPosition)*(sizeof(struct auth_data))));

	rewind(passwd);
	if (fwrite(passwdMem, (passwdSize - sizeof(struct auth_data)), 1, passwd) != 1)
		printf("Error writing data to passwd.\nExiting.\n");

	// truncate file
	fclose(passwd);
	passwd = 0;
	truncate(PASSWD, (passwdSize - sizeof(struct auth_data)));
	
	free(passwdMem);	
	CLEANUP:
	if (passwd != 0)
		fclose(passwd);
	return;

}	// DelUser



static void QueryPasswd(void) {

FILE *passwd;

	passwd = fopen(PASSWD, "r");
	if (passwd == 0)
      printf("Error opening passwd file: %s\n", PASSWD);
	else {
		printf("\nPasswd Entries:\n");
		printf("Key# MD5-Key                          Login\n");
		printf("---- -------------------------------- --------------------\n");
		while (fread(&passwdbuf, sizeof(struct auth_data), 1, passwd) == 1) { 
			printf ((char *)&passwdbuf);
		}
		printf("\n");
	}
	if (passwd)
		fclose(passwd);
}	// QueryPasswd



static void Help(void) {

	printf(help);

}	// Help
